@@ -53,6 +53,20 @@ def test_find_delimiter_pos_escapes():
     )
 
 
+def test_find_delimiter_pos_empty_and_whitespace():
+    """Test that empty and whitespace-only lines don't cause IndexError."""
+    # These should work for DialectJava (no key_wrap_char)
+    assert properties.DialectJava.find_delimiter("") == (None, -1)
+    assert properties.DialectJava.find_delimiter("   ") == (None, -1)
+    assert properties.DialectJava.find_delimiter("\t\t") == (None, -1)
+
+    # These should also work for DialectStrings (has key_wrap_char='"')
+    # This was causing IndexError before the fix
+    assert properties.DialectStrings.find_delimiter("") == (None, -1)
+    assert properties.DialectStrings.find_delimiter("   ") == (None, -1)
+    assert properties.DialectStrings.find_delimiter("\t\t") == (None, -1)
+
+
 def test_is_line_continuation():
     assert not properties.is_line_continuation("")
     assert not properties.is_line_continuation("some text")
@@ -169,7 +183,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         """Checks that a double properties definition can be regenerated as source."""
         propsource = "test_me=I can code!\ntest_me[one]=I can code single!"
         propregen = self.propregen(propsource).decode()
-        assert propsource + "\n" == propregen
+        assert f"{propsource}\n" == propregen
 
     def test_reduce(self):
         """Checks that if the target language has less plural form the generated properties file is correct."""
@@ -379,19 +393,19 @@ class TestProp(test_monolingual.TestMonolingualStore):
         """Checks that a simple properties definition can be regenerated as source."""
         propsource = "test_me=I can code!"
         propregen = self.propregen(propsource)
-        assert propsource + "\n" == propregen
+        assert f"{propsource}\n" == propregen
 
     def test_controlutf8_source(self):
         """Checks that a control characters are parsed correctly."""
         propsource = "test_me=\\\\\\n"
         propregen = self.propregen(propsource, encoding="utf-8")
-        assert propsource + "\n" == propregen
+        assert f"{propsource}\n" == propregen
 
     def test_control_source(self):
         """Checks that a control characters are parsed correctly."""
         propsource = "test_me=\\\\\\n"
         propregen = self.propregen(propsource)
-        assert propsource + "\n" == propregen
+        assert f"{propsource}\n" == propregen
 
     def test_unicode_escaping(self):
         """Check that escaped unicode is converted properly."""
@@ -410,7 +424,7 @@ class TestProp(test_monolingual.TestMonolingualStore):
         r"""Check that we preserve \n that appear at start and end of properties."""
         propsource = "newlines=\\ntext\\n"
         propregen = self.propregen(propsource)
-        assert propsource + "\n" == propregen
+        assert f"{propsource}\n" == propregen
 
     def test_space(self):
         r"""Check that we preserve \n that appear at start and end of properties."""
@@ -741,6 +755,23 @@ key=value
         assert bom not in result[3:]
         assert b"None" not in result[3:]
 
+    def test_utf16_bom_no_warning(self):
+        """Test that UTF-16 files with BOM do not trigger encoding warnings."""
+        import warnings
+
+        # Test UTF-16 with BOM (typical Mac .strings file)
+        propsource = r'"key" = "value";'.encode("utf-16")
+
+        # Parse should not trigger any warnings
+        with warnings.catch_warnings():
+            warnings.simplefilter("error")
+            propfile = self.propparse(propsource, personality="strings")
+
+        # Verify the file was parsed correctly
+        assert len(propfile.units) == 1
+        assert propfile.units[0].name == "key"
+        assert propfile.units[0].source == "value"
+
     def test_joomla_set_target(self):
         """Test various items used in Joomla files."""
         propsource = b"""COM_EXAMPLE_FOO="This is a test"\n"""
@@ -930,7 +961,7 @@ class TestXWiki(test_monolingual.TestMonolingualStore):
     def test_missing_definition_source(self):
         propsource = "### Missing: test_me=I can code!"
         propgen = self.propregen(propsource)
-        assert propsource + "\n" == propgen
+        assert f"{propsource}\n" == propgen
 
     def test_definition_with_simple_quote(self):
         propsource = "test_me=A 'quoted' translation"
@@ -940,7 +971,7 @@ class TestXWiki(test_monolingual.TestMonolingualStore):
         assert propunit.name == "test_me"
         assert propunit.source == "A 'quoted' translation"
         assert not propunit.missing
-        assert propunit.getoutput() == propsource + "\n"
+        assert propunit.getoutput() == f"{propsource}\n"
 
     def test_definition_with_simple_quote_and_argument(self):
         propsource = "test_me=A ''quoted'' translation for {0}"
@@ -950,17 +981,17 @@ class TestXWiki(test_monolingual.TestMonolingualStore):
         assert propunit.name == "test_me"
         assert propunit.source == "A 'quoted' translation for {0}"
         assert not propunit.missing
-        assert propunit.getoutput() == propsource + "\n"
+        assert propunit.getoutput() == f"{propsource}\n"
 
     def test_header_preserved(self):
         propsource = """# -----\n# Header\n# -----\n\ntest_me=I can code"""
         propgen = self.propregen(propsource)
-        assert propgen == propsource + "\n"
+        assert propgen == f"{propsource}\n"
 
     def test_blank_line_before_comment_preserved(self):
         propsource = """\n# My comment\ntest_me=I can code"""
         propgen = self.propregen(propsource)
-        assert propgen == propsource + "\n"
+        assert propgen == f"{propsource}\n"
 
     def test_deprecated_comments_preserved(self):
         propsource = """# Deprecated keys starts here.
@@ -1027,7 +1058,7 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
         generatedcontent = BytesIO()
         propfile.serialize(generatedcontent)
         assert (
-            generatedcontent.getvalue().decode(propfile.encoding) == propsource + "\n"
+            generatedcontent.getvalue().decode(propfile.encoding) == f"{propsource}\n"
         )
         # check translation and language attribute
         propfile.settargetlanguage("fr")
@@ -1065,7 +1096,7 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
     def test_missing_definition_source(self):
         propsource = self.getcontent("### Missing: test_me=I can code!")
         propgen = self.propregen(propsource)
-        assert propsource + "\n" == propgen
+        assert f"{propsource}\n" == propgen
 
     def test_definition_with_simple_quote(self):
         propsource = self.getcontent("test_me=A 'quoted' translation")
@@ -1078,7 +1109,7 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
         generatedcontent = BytesIO()
         propfile.serialize(generatedcontent)
         assert (
-            generatedcontent.getvalue().decode(propfile.encoding) == propsource + "\n"
+            generatedcontent.getvalue().decode(propfile.encoding) == f"{propsource}\n"
         )
 
     def test_definition_with_simple_quote_and_argument(self):
@@ -1092,7 +1123,7 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
         generatedcontent = BytesIO()
         propfile.serialize(generatedcontent)
         assert (
-            generatedcontent.getvalue().decode(propfile.encoding) == propsource + "\n"
+            generatedcontent.getvalue().decode(propfile.encoding) == f"{propsource}\n"
         )
 
     def test_definition_with_encoded_html(self):
@@ -1106,7 +1137,7 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
         generatedcontent = BytesIO()
         propfile.serialize(generatedcontent)
         assert (
-            generatedcontent.getvalue().decode(propfile.encoding) == propsource + "\n"
+            generatedcontent.getvalue().decode(propfile.encoding) == f"{propsource}\n"
         )
 
     def test_cleaning_attributes(self):
@@ -1118,9 +1149,7 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
         ## Real XWiki files are containing multiple attributes on xwikidoc tag: we're not testing it there
         ## because ElementTree changed its implementation between Python 3.7 and 3.8 which changed the order of output of the attributes
         ## it makes it more difficult to assert it on multiple versions of Python.
-        propsource = (
-            properties.XWikiPageProperties.XML_HEADER
-            + """<xwikidoc reference="XWiki.AdminTranslations">
+        propsource = f"""{properties.XWikiPageProperties.XML_HEADER}<xwikidoc reference="XWiki.AdminTranslations">
             <web>XWiki</web>
             <name>AdminTranslations</name>
             <language/>
@@ -1185,7 +1214,6 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
                 <content>something=toto</content>
             </attachment>
         </xwikidoc>"""
-        )
         propfile = self.propparse(propsource)
         assert len(propfile.units) == 1
         propunit = propfile.units[0]
@@ -1196,9 +1224,7 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
         propunit.target = "Je peux coder !"
         generatedcontent = BytesIO()
         propfile.serialize(generatedcontent)
-        expected_xml = (
-            properties.XWikiPageProperties.XML_HEADER
-            + """<xwikidoc reference="XWiki.AdminTranslations" locale="fr">
+        expected_xml = f"""{properties.XWikiPageProperties.XML_HEADER}<xwikidoc reference="XWiki.AdminTranslations" locale="fr">
             <web>XWiki</web>
             <name>AdminTranslations</name>
             <language>fr</language>
@@ -1218,9 +1244,8 @@ class TestXWikiPageProperties(test_monolingual.TestMonolingualStore):
 test_me=Je peux coder !
 </content>
 </xwikidoc>"""
-        )
         assert (
-            generatedcontent.getvalue().decode(propfile.encoding) == expected_xml + "\n"
+            generatedcontent.getvalue().decode(propfile.encoding) == f"{expected_xml}\n"
         )
         assert (
             '<?xml version="1.1" encoding="UTF-8"?>\n\n<!--\n * See the NOTICE file distributed with this work for additional'
@@ -1236,9 +1261,7 @@ test_me=Je peux coder !
         ## Real XWiki files are containing multiple attributes on xwikidoc tag: we're not testing it there
         ## because ElementTree changed its implementation between Python 3.7 and 3.8 which changed the order of output of the attributes
         ## it makes it more difficult to assert it on multiple versions of Python.
-        propsource = (
-            properties.XWikiPageProperties.XML_HEADER
-            + """<xwikidoc reference="XWiki.AdminTranslations">
+        propsource = f"""{properties.XWikiPageProperties.XML_HEADER}<xwikidoc reference="XWiki.AdminTranslations">
             <web>XWiki</web>
             <name>AdminTranslations</name>
             <language/>
@@ -1303,7 +1326,6 @@ test_me=Je peux coder !
                 <content>something=toto</content>
             </attachment>
         </xwikidoc>"""
-        )
         propfile = self.propparse(propsource)
         assert len(propfile.units) == 1
         propunit = propfile.units[0]
@@ -1314,9 +1336,7 @@ test_me=Je peux coder !
         propunit.target = "I can change the translation source"
         generatedcontent = BytesIO()
         propfile.serialize(generatedcontent)
-        expected_xml = (
-            properties.XWikiPageProperties.XML_HEADER
-            + """<xwikidoc reference="XWiki.AdminTranslations">
+        expected_xml = f"""{properties.XWikiPageProperties.XML_HEADER}<xwikidoc reference="XWiki.AdminTranslations">
             <web>XWiki</web>
             <name>AdminTranslations</name>
             <language/>
@@ -1381,9 +1401,8 @@ test_me=I can change the translation source
                 <content>something=toto</content>
             </attachment>
         </xwikidoc>"""
-        )
         assert (
-            generatedcontent.getvalue().decode(propfile.encoding) == expected_xml + "\n"
+            generatedcontent.getvalue().decode(propfile.encoding) == f"{expected_xml}\n"
         )
         assert (
             '<?xml version="1.1" encoding="UTF-8"?>\n\n<!--\n * See the NOTICE file distributed with this work for additional'
@@ -1393,15 +1412,12 @@ test_me=I can change the translation source
 
 class TestXWikiFullPage(test_monolingual.TestMonolingualStore):
     StoreClass = properties.XWikiFullPage
-    FILE_SCHEME = (
-        properties.XWikiPageProperties.XML_HEADER
-        + """<xwikidoc locale="%(language)s">
+    FILE_SCHEME = f"""{properties.XWikiPageProperties.XML_HEADER}<xwikidoc locale="%(language)s">
     <translation>1</translation>
     <language>%(language)s</language>
     <title>%(title)s</title>
     <content>%(content)s</content>
     </xwikidoc>"""
-    )
 
     def getcontent(self, content, title, language="en"):
         return self.FILE_SCHEME % {
@@ -1448,7 +1464,7 @@ class TestXWikiFullPage(test_monolingual.TestMonolingualStore):
         propfile.serialize(generatedcontent)
         assert (
             generatedcontent.getvalue().decode(propfile.encoding)
-            == expected_content + "\n"
+            == f"{expected_content}\n"
         )
 
     def test_parse(self):
@@ -1553,9 +1569,7 @@ class TestXWikiFullPage(test_monolingual.TestMonolingualStore):
         ## Real XWiki files are containing multiple attributes on xwikidoc tag: we're not testing it there
         ## because ElementTree changed its implementation between Python 3.7 and 3.8 which changed the order of output of the attributes
         ## it makes it more difficult to assert it on multiple versions of Python.
-        propsource = (
-            properties.XWikiPageProperties.XML_HEADER
-            + """<xwikidoc reference="XWiki.AdminTranslations">
+        propsource = f"""{properties.XWikiPageProperties.XML_HEADER}<xwikidoc reference="XWiki.AdminTranslations">
             <web>XWiki</web>
             <name>AdminTranslations</name>
             <language/>
@@ -1623,7 +1637,6 @@ class TestXWikiFullPage(test_monolingual.TestMonolingualStore):
                 <content>something=toto</content>
             </attachment>
         </xwikidoc>"""
-        )
         propfile = self.propparse(propsource)
         assert len(propfile.units) == 2
         propunit = propfile.units[0]
@@ -1654,9 +1667,7 @@ class TestXWikiFullPage(test_monolingual.TestMonolingualStore):
         propfile.settargetlanguage("fr")
         propfile.serialize(generatedcontent)
 
-        expected_xml = (
-            properties.XWikiPageProperties.XML_HEADER
-            + """<xwikidoc reference="XWiki.AdminTranslations" locale="fr">
+        expected_xml = f"""{properties.XWikiPageProperties.XML_HEADER}<xwikidoc reference="XWiki.AdminTranslations" locale="fr">
             <web>XWiki</web>
             <name>AdminTranslations</name>
             <language>fr</language>
@@ -1679,9 +1690,8 @@ class TestXWikiFullPage(test_monolingual.TestMonolingualStore):
             D'autres trucs.
             </content>
             </xwikidoc>"""
-        )
         assert (
-            generatedcontent.getvalue().decode(propfile.encoding) == expected_xml + "\n"
+            generatedcontent.getvalue().decode(propfile.encoding) == f"{expected_xml}\n"
         )
 
     @mark.xfail(reason="removal not working in full page")
