@@ -229,7 +229,17 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser):
     def processfile(
         self, fileprocessor, options, fullinputpath, fulloutputpath, fulltemplatepath
     ):
-        if options.timestamp and _output_is_newer(fullinputpath, fulloutputpath):
+        # The `fulloutputpath`s parsed when `onefile` is enabled don't exist, which causes
+        # the `_output_is_newer` check to return False. This means that even if there is
+        # no content update, the POT/PO files are regenerated. When `onefile` is enabled,
+        # this check sets the output path used by `_output_is_newer` to the output file
+        # provided to the command, which will exist. It remains `fulloutputpath` otherwise.
+        if getattr(options, "multifilestyle", None) == "onefile":
+            timecheck_output_path = options.output
+        else:
+            timecheck_output_path = fulloutputpath
+
+        if options.timestamp and _output_is_newer(fullinputpath, timecheck_output_path):
             return False
 
         return super().processfile(
@@ -402,14 +412,15 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
 
     def opentemplatefile(self, options, fulltemplatepath):
         """Opens the template file (if required)."""
-        if fulltemplatepath is not None:
-            if options.recursivetemplate and self.isarchive(
-                options.template, "template"
-            ):
-                # TODO: deal with different names in input/template archives
-                if fulltemplatepath in self.templatearchive:
-                    return self.templatearchive.openinputfile(fulltemplatepath)
-                self.warning(f"missing template file {fulltemplatepath}")
+        if (
+            fulltemplatepath is not None
+            and options.recursivetemplate
+            and self.isarchive(options.template, "template")
+        ):
+            # TODO: deal with different names in input/template archives
+            if fulltemplatepath in self.templatearchive:
+                return self.templatearchive.openinputfile(fulltemplatepath)
+            self.warning(f"missing template file {fulltemplatepath}")
         return super().opentemplatefile(options, fulltemplatepath)
 
     def getfulltemplatepath(self, options, templatepath):
