@@ -22,16 +22,19 @@ from __future__ import annotations
 
 import codecs
 import logging
-import pickle
 from io import BytesIO
 from itertools import starmap
-from typing import Callable, ClassVar, Literal
+from typing import TYPE_CHECKING, ClassVar, Literal
 
+from translate.lang.data import plural_tags
 from translate.misc.multistring import multistring
 from translate.storage.placeables import StringElem
 from translate.storage.placeables import parse as rich_parse
 from translate.storage.workflow import StateEnum as states
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
 logger = logging.getLogger(__name__)
 
 # Simple BOM based encoding detection
@@ -325,8 +328,7 @@ class TranslationUnit:
         else:
             self.addlocation(location)
 
-    @staticmethod
-    def getcontext():
+    def getcontext(self):
         """Get the message context."""
         return ""
 
@@ -357,7 +359,7 @@ class TranslationUnit:
                        - 'developer', 'programmer', 'source code' (synonyms)
         """
         if position == "append" and getattr(self, "notes", None):
-            self.notes += "\n" + text
+            self.notes += f"\n{text}"
         else:
             self.notes = text
 
@@ -716,11 +718,7 @@ class TranslationStore:
             remove_source(unit.source)
 
         for location in unit.getlocations():
-            if (
-                location in self.locationindex
-                and self.locationindex[location] is not None
-                and self.locationindex[location] == unit
-            ):
+            if location in self.locationindex and self.locationindex[location] == unit:
                 del self.locationindex[location]
 
     def add_unit_to_index(self, unit):
@@ -785,8 +783,16 @@ class TranslationStore:
         Converts to a bytes representation that can be parsed back using
         :meth:`~.TranslationStore.parsestring`.
         `out` should be an open file-like objects to write to.
+
+        .. note::
+
+           This method should be overridden by subclasses to provide
+           format-specific serialization.
         """
-        out.write(pickle.dumps(self))
+        raise NotImplementedError(
+            f"{self.__class__.__name__} does not implement serialize(). "
+            "Subclasses must provide their own serialization method."
+        )
 
     def isempty(self):
         """Return True if the object doesn't contain any translation units."""
@@ -894,8 +900,18 @@ class TranslationStore:
         return r_text, r_encoding
 
     def parse(self, data):
-        """Parser to process the given source string."""
-        self.units = pickle.loads(data).units
+        """
+        Parser to process the given source string.
+
+        .. note::
+
+           This method should be overridden by subclasses to provide
+           format-specific parsing.
+        """
+        raise NotImplementedError(
+            f"{self.__class__.__name__} does not implement parse(). "
+            "Subclasses must provide their own parsing method."
+        )
 
     def savefile(self, storefile):
         """Write the string representation to the given file (or filename)."""
@@ -959,6 +975,19 @@ class TranslationStore:
         """
         return "id"
 
+    def get_base_locale_code(self) -> str:
+        locale = self.gettargetlanguage()
+        if not locale:
+            return "en"
+        # Handle b+ style language codes and standardize
+        return (
+            locale.removeprefix("b+").replace("_", "-").replace("+", "-").split("-")[0]
+        )
+
+    def get_plural_tags(self) -> list[str]:
+        locale = self.get_base_locale_code()
+        return plural_tags.get(locale, plural_tags["en"])
+
 
 class UnitId:
     KEY_SEPARATOR = "."
