@@ -113,8 +113,7 @@ class pounit(pocommon.pounit):
                 self._target = target[0]
             else:
                 raise ValueError(
-                    "po msgid element has no plural but msgstr"
-                    "has %d elements (%s)" % (len(target), target)
+                    f"po msgid element has no plural but msgstr has {len(target)} elements ({target})"
                 )
         else:
             self._target = target
@@ -241,7 +240,7 @@ class pounit(pocommon.pounit):
             # Remove kde-style comments from the translation (if any). XXX - remove
             if pocommon.extract_msgid_comment(otherpo.target):
                 otherpo.target = otherpo.target.replace(
-                    "_: " + otherpo._extract_msgidcomments() + "\n", ""
+                    f"_: {otherpo._extract_msgidcomments()}\n", ""
                 )
             self.target = otherpo.target
             if (
