@@ -285,7 +285,7 @@ class Dialect:
         # FIXME: dialects are a bad idea, not possible for subclasses
         # to override key methods
         if encoding not in {"utf-8", "utf-16"}:
-            return javapropertiesencode(string or "")
+            return javapropertiesencode(string or "", encoding)
         return java_utf8_properties_encode(string or "")
 
     @staticmethod
@@ -314,7 +314,11 @@ class Dialect:
         delimiters = delimiter_dict
         # Figure out starting position
         start_pos = len(line) - len(line.lstrip())  # Skip initial whitespace
-        if cls.key_wrap_char and line[start_pos] == cls.key_wrap_char:
+        if (
+            cls.key_wrap_char
+            and start_pos < len(line)
+            and line[start_pos] == cls.key_wrap_char
+        ):
             # Skip the key if it is delimited by some char
             start_pos += 1
             while line[start_pos] != cls.key_wrap_char or line[start_pos - 1] == "\\":
@@ -339,16 +343,16 @@ class Dialect:
         if mindelimiter is None and delimiters.get(" ", -1) != -1:
             # Use space delimiter if we found nothing else
             return (" ", delimiters[" "])
+        # If space delimiter occurs earlier than ":" or "=" then it is the
+        # delimiter only if there are non-whitespace characters between it and
+        # the other detected delimiter.
         if (
             mindelimiter is not None
             and " " in delimiters
             and delimiters[" "] < delimiters[mindelimiter]
+            and len(line[delimiters[" "] : delimiters[mindelimiter]].strip()) > 0
         ):
-            # If space delimiter occurs earlier than ":" or "=" then it is the
-            # delimiter only if there are non-whitespace characters between it and
-            # the other detected delimiter.
-            if len(line[delimiters[" "] : delimiters[mindelimiter]].strip()) > 0:
-                return (" ", delimiters[" "])
+            return (" ", delimiters[" "])
         return (mindelimiter, minpos)
 
     @staticmethod
@@ -579,8 +583,8 @@ class DialectStrings(Dialect):
 
     @staticmethod
     def is_line_continuation(line):
-        l = line.rstrip()
-        return not l or l[-1] != ";"
+        stripped = line.rstrip()
+        return not stripped or stripped[-1] != ";"
 
     @staticmethod
     def strip_line_continuation(value):
@@ -614,12 +618,6 @@ class proppluralunit(base.TranslationUnit):
         existing = self.units.keys()
         return [key for key in data.cldr_plural_categories if key in existing]
 
-    def _get_target_mapping(self):
-        cldr_mapping = proppluralunit._get_language_mapping(self._store.targetlanguage)
-        if cldr_mapping:
-            return cldr_mapping
-        return self._get_existing_mapping()
-
     def _get_source_mapping(self):
         cldr_mapping = proppluralunit._get_language_mapping(self._store.sourcelanguage)
         if cldr_mapping:
@@ -659,7 +657,7 @@ class proppluralunit(base.TranslationUnit):
     def _get_ordered_units(self):
         # Used for str (GWT order)
         mapping = self.personality.get_expand_output_target_mapping(
-            self._get_target_mapping()
+            self._store.get_plural_tags()
         )
         names = [
             name for name in self.personality.get_cldr_names_order() if name in mapping
@@ -678,11 +676,11 @@ class proppluralunit(base.TranslationUnit):
         elif isinstance(text, list):
             strings = text
         elif isinstance(text, dict):
-            mapping, strings = map(list, zip(*text.items()))
+            mapping, strings = map(list, zip(*text.items(), strict=True))
         else:
             strings = [text]
         if mapping is None:
-            mapping = self._get_target_mapping()
+            mapping = self._store.get_plural_tags()
 
         strings = self._get_strings(strings, mapping)
         units = self._get_units(mapping)
@@ -693,11 +691,11 @@ class proppluralunit(base.TranslationUnit):
         if "other" not in mapping and "other" in self.units:
             self.units["other"].target = strings[-1]
 
-        for a, b in zip(strings, units):
+        for a, b in zip(strings, units, strict=True):
             b.target = a
 
     def gettarget(self):
-        ll = [x.target for x in self._get_units(self._get_target_mapping())]
+        ll = [x.target for x in self._get_units(self._store.get_plural_tags())]
         if len(ll) > 1:
             return multistring(ll)
         return ll[0]
@@ -717,7 +715,7 @@ class proppluralunit(base.TranslationUnit):
         elif isinstance(text, list):
             strings = text
         elif isinstance(text, dict):
-            mapping, strings = tuple(map(list, zip(*text.items())))
+            mapping, strings = tuple(map(list, zip(*text.items(), strict=True)))
         else:
             strings = [text]
         if mapping is None:
@@ -730,7 +728,7 @@ class proppluralunit(base.TranslationUnit):
                 f'Not same plural counts between "{strings}" and "{units}"'
             )
 
-        for a, b in zip(strings, units):
+        for a, b in zip(strings, units, strict=True):
             b.source = a
 
     source = property(getsource, setsource)
