@@ -1,6 +1,7 @@
 #
 # Copyright 2002-2009 Zuza Software Foundation
 # Copyright 2013 F Wolff
+# Copyright 2024 gemmaro <gemmaro.dev@gmail.com>
 #
 # This file is part of the Translate Toolkit.
 #
@@ -17,19 +18,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""Classes that hold units of Gettext .po files (pounit) or entire
+"""
+Classes that hold units of Gettext .po files (pounit) or entire
 files (pofile).
 """
 
+from __future__ import annotations
+
 import copy
 import logging
 import re
-import textwrap
+from itertools import chain
+from string import punctuation
+from typing import TYPE_CHECKING
+
+from unicode_segmentation_rs import split_word_bounds, text_width
 
 from translate.misc import quote
 from translate.misc.multistring import multistring
 from translate.storage import pocommon, poparser
 
+if TYPE_CHECKING:
+    from collections.abc import Iterable
 
 logger = logging.getLogger(__name__)
 
@@ -39,12 +49,26 @@ lsep = "\n#: "
 
 # general functions for quoting / unquoting po strings
 
-po_unescape_map = {"\\r": "\r", "\\t": "\t", '\\"': '"', "\\n": "\n", "\\\\": "\\"}
+po_unescape_map = {
+    r"\r": "\r",
+    r"\t": "\t",
+    r"\"": '"',
+    r"\n": "\n",
+    r"\\": "\\",
+}
+po_unescape_re = re.compile(r"\\.")
 po_escape_map = {value: key for (key, value) in po_unescape_map.items()}
+po_escape_re = re.compile("|".join(re.escape(key) for key in po_escape_map))
+
+po_line_break_chars = {"/", "}", ")", ">", "-"}
+po_mergeable_chars = po_line_break_chars | {" ", "\t"}
+po_open_parenthesis_chars = {"{", "("}
+po_punctuation = set(punctuation)
 
 
 def splitlines(text):
-    """Split lines based on first newline char.
+    """
+    Split lines based on first newline char.
 
     Can not use univerzal newlines as they match any newline like
     character inside text and that breaks on files with unix newlines
@@ -56,7 +80,7 @@ def splitlines(text):
     """
     # Strip UTF-8 BOM if present. This file would not be accepted
     # by gettext, but some editors might create it, so better handle it.
-    if text[:3] == b"\xEF\xBB\xBF":
+    if text[:3] == b"\xef\xbb\xbf":
         text = text[3:]
     # Find first newline
     newline = b"\n"
@@ -66,82 +90,151 @@ def splitlines(text):
         if ch == 10:
             break
         if ch == 13:
-            if text[msgid_pos + i + 1] == 10:
-                newline = b"\r\n"
-            else:
-                newline = b"\r"
+            newline = b"\r\n" if text[msgid_pos + i + 1] == 10 else b"\r"
             break
 
     return [x + newline for x in text.split(newline)], newline.decode()
 
 
-def escapeforpo(line):
-    """Escapes a line for po format. assumes no \n occurs in the line.
+def escapehandler(match: re.Match) -> str:
+    return po_escape_map[match.group(0)]
+
+
+def escapeforpo(line: str) -> str:
+    r"""
+    Escapes a line for po format. assumes no \n occurs in the line.
 
     :param line: unescaped text
     """
-    special_locations = []
-    for special_key in po_escape_map:
-        special_locations.extend(quote.find_all(line, special_key))
-    special_locations = sorted(dict.fromkeys(special_locations).keys())
-    escaped_line = []
-    last_location = 0
-    for location in special_locations:
-        escaped_line.append(line[last_location:location])
-        escaped_line.append(po_escape_map[line[location : location + 1]])
-        last_location = location + 1
-    escaped_line.append(line[last_location:])
-    return "".join(escaped_line)
+    return po_escape_re.sub(escapehandler, line)
 
 
-def unescapehandler(escape):
-    return po_unescape_map.get(escape, escape)
+class PoWrapper:
+    r"""
+    Gettext-compatible text wrapper for PO files.
+
+    This implementation follows gettext's wrapping behavior:
+    - Never breaks escape sequences (\\n, \\", etc.)
+    - Prefers breaking after spaces
+    - Handles CJK characters with proper width calculation
+    - Breaks long words only when necessary
+    """
+
+    def __init__(self, width: int = 77) -> None:
+        self.width = width
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, PoWrapper):
+            return False
+        return self.width == other.width
+
+    def wrap(self, text: str) -> list[str]:
+        """
+        Wrap text to fit within the specified width.
+
+        Returns a list of lines, each no longer than self.width characters
+        (accounting for character display width for CJK characters).
+        """
+        if not text or self.width <= 0:
+            return [text] if text else []
+
+        # Split text into chunks using word separator pattern
+        chunks = self._split_chunks(text)
+
+        # Now wrap the chunks into lines
+        return self._wrap_chunks(chunks)
+
+    def _split_chunks(self, text: str) -> list[list[str]]:
+        """
+        Split text into chunks at word boundaries using Unicode segmentation.
+
+        Uses the unicode_segmentation_py to split to words and postprocess
+        to apply gettext specific rules.
+        """
+        chunks: list[list[str]] = []
+        last_char = second_last_char = ""
+        last_chunk: list[str] = []
+
+        for chunk in split_word_bounds(text):
+            if (
+                last_char
+                and (
+                    not second_last_char
+                    or (last_char != "\\" or second_last_char != "\\")
+                )
+                and (
+                    chunk in po_mergeable_chars
+                    or (
+                        chunk not in po_open_parenthesis_chars
+                        and last_char not in po_line_break_chars
+                        and (
+                            last_char in po_punctuation
+                            or (chunk in po_punctuation and not last_char.isspace())
+                        )
+                    )
+                )
+            ):
+                last_chunk.append(chunk)
+                second_fallback = last_char
+            else:
+                last_chunk = [chunk]
+                chunks.append(last_chunk)
+                # In case the chunk is 1 char, we don't have previous character
+                second_fallback = ""
+
+            try:
+                second_last_char = chunk[-2]
+                last_char = chunk[-1]
+            except IndexError:
+                second_last_char = second_fallback
+                last_char = chunk
+
+        return chunks
+
+    def _wrap_chunks(self, chunks: Iterable[list[str]]) -> list[str]:
+        """Wrap chunks into lines."""
+        lines = []
+        current_line = []
+        current_width = 0
+
+        for chunk in chunks:
+            chunk_width = sum(text_width(part) for part in chunk)
+
+            # Try to add chunk to current line
+            if current_width + chunk_width <= self.width:
+                current_line.extend(chunk)
+                current_width += chunk_width
+            # Chunk doesn't fit
+            else:
+                if current_line:
+                    # Save current line and start new one
+                    lines.append("".join(current_line))
+                    current_line = []
+                    current_width = 0
+
+                # Add the whole chunk as one line
+                current_line.extend(chunk)
+                current_width += chunk_width
+
+        # Add remaining line
+        if current_line:
+            lines.append("".join(current_line))
+
+        return lines
 
 
-class PoWrapper(textwrap.TextWrapper):
-    wordsep_re = re.compile(
-        r"""
-            (
-            \s+|                                  # any whitespace
-            [a-z0-9A-Z_-]+/|                      # nicely split long URLs
-            \w*\\.\w*|                            # any escape should not be split
-            n(?=%)|                               # wrap insidide plural equation
-            \.(?=\w)|                             # full stop inside word
-            [\w\!\'\&\.\,\?=<>%]+\s+|             # space should go with a word
-            [^\s\w]*\w+[a-zA-Z]-(?=\w+[a-zA-Z])|  # hyphenated words
-            (?<=[\w\!\"\'\&\.\,\?])-{2,}(?=\w)    # em-dash
-            )
-        """,
-        re.VERBOSE,
-    )
-
-    def __init__(
-        self,
-        width=77,
-        replace_whitespace=False,
-        expand_tabs=False,
-        drop_whitespace=False,
-    ):
-        super().__init__(
-            width=width,
-            replace_whitespace=replace_whitespace,
-            expand_tabs=expand_tabs,
-            drop_whitespace=drop_whitespace,
-        )
-
-
-def quoteforpo(text, wrapper_obj=None):
-    """Quotes the given text for a PO file, returning quoted and escaped lines"""
+def quoteforpo(text: str | None, wrapper_obj: PoWrapper | None = None) -> list[str]:
+    """Quotes the given text for a PO file, returning quoted and escaped lines."""
     if text is None:
         return []
     if wrapper_obj is None:
         wrapper_obj = PoWrapper()
     text = escapeforpo(text)
     if wrapper_obj.width == -1:
-        return ['"%s"' % text]
+        return [f'"{text}"']
     lines = text.split("\\n")
-    for i, l in enumerate(lines[:-1]):
-        lines[i] = l + "\\n"
+    for i, line in enumerate(lines[:-1]):
+        lines[i] = f"{line}\\n"
 
     polines = []
     len_lines = len(lines)
@@ -153,53 +246,37 @@ def quoteforpo(text, wrapper_obj=None):
         polines.append('""')
     for line in lines:
         lns = wrapper_obj.wrap(line)
-        for ln in lns:
-            polines.append('"%s"' % ln)
+        polines.extend(f'"{ln}"' for ln in lns)
     return polines
 
 
-def unescape(line):
-    """Unescape the given line.
+def unescapehandler(match: re.Match) -> str:
+    value = match.group(0)
+    try:
+        return po_unescape_map[value]
+    except KeyError:
+        return value[1]
+
+
+def unescape(line: str) -> str:
+    """
+    Unescape the given line.
 
     Quotes on either side should already have been removed.
     """
-    escape_places = quote.find_all(line, "\\")
-    if not escape_places:
-        return line
-
-    # filter escaped escapes
-    true_escape = False
-    true_escape_places = []
-    for escape_pos in escape_places:
-        if escape_pos - 1 in escape_places:
-            true_escape = not true_escape
-        else:
-            true_escape = True
-        if true_escape:
-            true_escape_places.append(escape_pos)
-
-    extracted = []
-    lastpos = 0
-    for pos in true_escape_places:
-        # everything leading up to the escape
-        extracted.append(line[lastpos:pos])
-        # the escaped sequence (consuming 2 characters)
-        extracted.append(unescapehandler(line[pos : pos + 2]))
-        lastpos = pos + 2
-
-    extracted.append(line[lastpos:])
-    return "".join(extracted)
+    return po_unescape_re.sub(unescapehandler, line)
 
 
-def unquotefrompo(postr):
+def unquotefrompo(postr: list[str]) -> str:
     return "".join(unescape(line[1:-1]) for line in postr)
 
 
-def is_null(lst):
-    return lst == [] or len(lst) == 1 and lst[0] == '""'
+def is_null(lst: list[str]) -> bool:
+    lst_len = len(lst)
+    return lst_len == 0 or (lst_len == 1 and lst[0] == '""')
 
 
-def extractstr(string):
+def extractstr(string: str) -> str:
     left = string.find('"')
     right = string.rfind('"')
     if right > -1:
@@ -224,18 +301,18 @@ class pounit(pocommon.pounit):
     # fashion
     __shallow__ = ["_store", "wrapper"]
 
-    def __init__(self, source=None, wrapper=None, **kwargs):
-        self.wrapper = wrapper
-        self.obsolete = False
+    def __init__(self, source=None, wrapper: PoWrapper | None = None, **kwargs):
+        self.wrapper: PoWrapper | None = wrapper
+        self.obsolete: bool = False
         self._initallcomments(blankall=True)
-        self.prev_msgctxt = []
-        self.prev_msgid = []
-        self.prev_msgid_plural = []
-        self.msgctxt = []
-        self.msgid = []
-        self.msgid_pluralcomments = []
-        self.msgid_plural = []
-        self.msgstr = []
+        self.prev_msgctxt: list[str] = []
+        self.prev_msgid: list[str] = []
+        self.prev_msgid_plural: list[str] = []
+        self.msgctxt: list[str] = []
+        self.msgid: list[str] = []
+        self.msgid_pluralcomments: list[str] = []
+        self.msgid_plural: list[str] = []
+        self.msgstr: list[str] | dict[int, list[str]] = []
         super().__init__(source)
 
     @property
@@ -245,7 +322,7 @@ class pounit(pocommon.pounit):
         return "\n"
 
     def _initallcomments(self, blankall=False):
-        """Initialises allcomments"""
+        """Initialises allcomments."""
         if blankall:
             self.othercomments = []
             self.automaticcomments = []
@@ -271,33 +348,29 @@ class pounit(pocommon.pounit):
             return multistring([singular, pluralform])
         return singular
 
-    def quote(self, text):
+    def quote(self, text: str) -> list[str]:
         return quoteforpo(text, self.wrapper)
 
-    def _set_source_vars(self, source):
-        msgid = None
-        msgid_plural = None
+    def _set_source_vars(
+        self, source: str | list[str] | multistring
+    ) -> tuple[list[str], list[str]]:
         if isinstance(source, multistring):
             source = source.strings
         if isinstance(source, list):
-            msgid = self.quote(source[0])
-            if len(source) > 1:
-                msgid_plural = self.quote(source[1])
-            else:
-                msgid_plural = []
-        else:
-            msgid = self.quote(source)
-            msgid_plural = []
-        return msgid, msgid_plural
+            return self.quote(source[0]), self.quote(source[1]) if len(
+                source
+            ) > 1 else []
+        return self.quote(source), []
 
     @property
     def source(self):
-        """Returns the unescaped msgid"""
+        """Returns the unescaped msgid."""
         return self._get_source_vars(self.msgid, self.msgid_plural)
 
     @source.setter
-    def source(self, source):
-        """Sets the msgid to the given (unescaped) value.
+    def source(self, source: str | list[str] | multistring) -> None:
+        """
+        Sets the msgid to the given (unescaped) value.
 
         :param source: an unescaped source string.
         """
@@ -305,11 +378,12 @@ class pounit(pocommon.pounit):
         self.msgid, self.msgid_plural = self._set_source_vars(source)
 
     def _get_prev_source(self):
-        """Returns the unescaped msgid"""
+        """Returns the unescaped msgid."""
         return self._get_source_vars(self.prev_msgid, self.prev_msgid_plural)
 
     def _set_prev_source(self, source):
-        """Sets the msgid to the given (unescaped) value.
+        """
+        Sets the msgid to the given (unescaped) value.
 
         :param source: an unescaped source string.
         """
@@ -319,14 +393,14 @@ class pounit(pocommon.pounit):
 
     @property
     def target(self):
-        """Returns the unescaped msgstr"""
+        """Returns the unescaped msgstr."""
         if isinstance(self.msgstr, dict):
             return multistring(list(map(unquotefrompo, self.msgstr.values())))
         return unquotefrompo(self.msgstr)
 
     @target.setter
     def target(self, target):
-        """Sets the msgstr to the given (unescaped) value"""
+        """Sets the msgstr to the given (unescaped) value."""
         self._rich_target = None
         if self.hasplural():
             if isinstance(target, multistring):
@@ -338,8 +412,7 @@ class pounit(pocommon.pounit):
                 target = target[0]
             else:
                 raise ValueError(
-                    "po msgid element has no plural but msgstr has %d elements (%s)"
-                    % (len(target), target)
+                    f"po msgid element has no plural but msgstr has {len(target)} elements ({target})"
                 )
         templates = self.msgstr
         if isinstance(templates, list):
@@ -354,7 +427,8 @@ class pounit(pocommon.pounit):
             self.msgstr = self.quote(target)
 
     def getalttrans(self):
-        """Return a list of alternate units.
+        """
+        Return a list of alternate units.
 
         Previous msgid and current msgstr is combined to form a single
         alternative unit.
@@ -363,40 +437,30 @@ class pounit(pocommon.pounit):
         if prev_source and self.isfuzzy():
             unit = type(self)(prev_source)
             unit.target = self.target
-            # Already released versions of Virtaal (0.6.x) only supported XLIFF
-            # alternatives, and expect .xmlelement.get().
-            # This can be removed soon:
-            unit.xmlelement = dict()
             return [unit]
         return []
 
-    def getnotes(self, origin=None):
-        """Return comments based on origin value.
+    def getnotes(self, origin: str | None = None) -> str:
+        """
+        Return comments based on origin value.
 
         :param origin: programmer, developer, source code, translator or None
         """
-        if origin is None:
-            comments = "".join(
-                comment[2:] or self.newline for comment in self.othercomments
-            )
-            comments += "".join(
-                comment[3:] or self.newline for comment in self.automaticcomments
-            )
-        elif origin == "translator":
-            comments = "".join(
-                comment[2:] or self.newline for comment in self.othercomments
-            )
-        elif origin in ["programmer", "developer", "source code"]:
-            comments = "".join(
-                comment[3:] or self.newline for comment in self.automaticcomments
-            )
-        else:
+        parts = []
+        newline = self.newline
+        if origin == "translator" or origin is None:
+            parts.append(comment[2:] or newline for comment in self.othercomments)
+        if origin in {"programmer", "developer", "source code", None}:
+            parts.append(comment[3:] or newline for comment in self.automaticcomments)
+        if not parts:
             raise ValueError("Comment type not valid")
+        comments = "".join(chain.from_iterable(parts))
         # Let's drop the last newline
-        return comments[:-1]
+        return comments[: -len(newline)]
 
-    def addnote(self, text, origin=None, position="append"):
-        """This is modeled on the XLIFF method.
+    def addnote(self, text: str, origin: str | None = None, position: str = "append"):
+        """
+        This is modeled on the XLIFF method.
 
         See :meth:`translate.storage.xliff.xliffunit.addnote`
         """
@@ -406,7 +470,7 @@ class pounit(pocommon.pounit):
         commentlist = self.othercomments
         linestart = "#"
         autocomments = False
-        if origin in ["programmer", "developer", "source code"]:
+        if origin in {"programmer", "developer", "source code"}:
             autocomments = True
             commentlist = self.automaticcomments
             linestart = "#."
@@ -417,7 +481,7 @@ class pounit(pocommon.pounit):
         if position == "append":
             newcomments = commentlist + newcomments
         elif position == "prepend":
-            newcomments = newcomments + commentlist
+            newcomments += commentlist
 
         if autocomments:
             self.automaticcomments = newcomments
@@ -425,7 +489,7 @@ class pounit(pocommon.pounit):
             self.othercomments = newcomments
 
     def removenotes(self, origin=None):
-        """Remove all the translator's notes (other comments)"""
+        """Remove all the translator's notes (other comments)."""
         self.othercomments = []
 
     def __deepcopy__(self, memo={}):
@@ -459,14 +523,12 @@ class pounit(pocommon.pounit):
 
     def _msgstrlen(self):
         if isinstance(self.msgstr, dict):
-            combinedstr = self.newline.join(
-                filter(None, [unquotefrompo(msgstr) for msgstr in self.msgstr.values()])
-            )
-            return len(combinedstr)
+            return sum(len(unquotefrompo(msgstr)) for msgstr in self.msgstr.values())
         return len(unquotefrompo(self.msgstr))
 
-    def merge(self, otherpo, overwrite=False, comments=True, authoritative=False):
-        """Merges the otherpo (with the same msgid) into this one.
+    def merge(self, otherunit, overwrite=False, comments=True, authoritative=False):
+        """
+        Merges the otherunit (with the same msgid) into this one.
 
         Overwrite non-blank self.msgstr only if overwrite is True
         merge comments only if comments is True
@@ -502,49 +564,47 @@ class pounit(pocommon.pounit):
                         if item not in splitlist1
                     ]
                 )
-            else:
+            elif list1 != list2:
                 # Normal merge, but conform to list1 newline style
-                if list1 != list2:
-                    for item in list2:
-                        if lineend:
-                            item = item.rstrip() + lineend
-                        # avoid duplicate comment lines (this might cause some problems)
-                        if item not in list1 or len(item) < 5:
-                            list1.append(item)
+                for item in list2:
+                    if lineend:
+                        item = item.rstrip() + lineend
+                    # avoid duplicate comment lines (this might cause some problems)
+                    if item not in list1 or len(item) < 5:
+                        list1.append(item)
 
-        if not isinstance(otherpo, pounit):
-            super().merge(otherpo, overwrite, comments)
+        if not isinstance(otherunit, pounit):
+            super().merge(otherunit, overwrite, comments)
             return
         if comments:
-            mergelists(self.othercomments, otherpo.othercomments)
-            mergelists(self.typecomments, otherpo.typecomments)
+            mergelists(self.othercomments, otherunit.othercomments)
+            mergelists(self.typecomments, otherunit.typecomments)
             if not authoritative:
-                # We don't bring across otherpo.automaticcomments as we
+                # We don't bring across otherunit.automaticcomments as we
                 # consider ourself to be the the authority.  Same applies
-                # to otherpo.msgidcomments
-                mergelists(self.automaticcomments, otherpo.automaticcomments)
-                mergelists(self.msgidcomments, otherpo.msgidcomments)
-                mergelists(self.sourcecomments, otherpo.sourcecomments, split=True)
+                # to otherunit.msgidcomments
+                mergelists(self.automaticcomments, otherunit.automaticcomments)
+                mergelists(self.msgidcomments, otherunit.msgidcomments)
+                mergelists(self.sourcecomments, otherunit.sourcecomments, split=True)
         if not self.istranslated() or overwrite:
             # Remove kde-style comments from the translation (if any).
-            if self._extract_msgidcomments(otherpo.target):
-                otherpo.target = otherpo.target.replace(
-                    "_: " + otherpo._extract_msgidcomments() + self.newline, ""
+            if self._extract_msgidcomments(otherunit.target):
+                otherunit.target = otherunit.target.replace(
+                    f"_: {otherunit._extract_msgidcomments()}{self.newline}", ""
                 )
-            self.target = otherpo.target
+            self.target = otherunit.target
             if (
-                self.source != otherpo.source
-                or self.getcontext() != otherpo.getcontext()
+                self.source != otherunit.source
+                or self.getcontext() != otherunit.getcontext()
             ):
                 self.markfuzzy()
             else:
-                self.markfuzzy(otherpo.isfuzzy())
-        elif not otherpo.istranslated():
-            if self.source != otherpo.source:
-                self.markfuzzy()
-        else:
-            if self.target != otherpo.target:
+                self.markfuzzy(otherunit.isfuzzy())
+        elif not otherunit.istranslated():
+            if self.source != otherunit.source:
                 self.markfuzzy()
+        elif self.target != otherunit.target:
+            self.markfuzzy()
 
     def isheader(self):
         # return (self._msgidlen() == 0) and (self._msgstrlen() > 0) and (len(self.msgidcomments) == 0)
@@ -552,20 +612,18 @@ class pounit(pocommon.pounit):
         return (
             is_null(self.msgid)
             and not is_null(self.msgstr)
-            and self.msgidcomments == []
+            and len(self.msgidcomments) == 0
             and is_null(self.msgctxt)
         )
 
     def isblank(self):
-        if self.isheader() or self.msgidcomments:
-            return False
-        if (
-            (self._msgidlen() == 0)
+        return (
+            not self.isheader()
+            and not self.msgidcomments
+            and (self._msgidlen() == 0)
             and (self._msgstrlen() == 0)
             and (is_null(self.msgctxt))
-        ):
-            return True
-        return False
+        )
         # TODO: remove:
         # Before, the equivalent of the following was the final return statement:
         # return len(self.source.strip()) == 0
@@ -579,7 +637,7 @@ class pounit(pocommon.pounit):
                 yield value
 
     def hastypecomment(self, typecomment, parsed=None):
-        """Check whether the given type comment is present"""
+        """Check whether the given type comment is present."""
         if not self.typecomments:
             return False
         if not parsed:
@@ -587,20 +645,21 @@ class pounit(pocommon.pounit):
         return typecomment in parsed
 
     def hasmarkedcomment(self, commentmarker):
-        """Check whether the given comment marker is present.
+        """
+        Check whether the given comment marker is present.
 
         These should appear as::
 
                 # (commentmarker) ...
         """
-        commentmarker = "(%s)" % commentmarker
+        commentmarker = f"({commentmarker})"
         for comment in self.othercomments:
             if comment.replace("#", "", 1).strip().startswith(commentmarker):
                 return True
         return False
 
     def settypecomment(self, typecomment, present=True):
-        """Alters whether a given typecomment is present"""
+        """Alters whether a given typecomment is present."""
         typecomments = list(self._extracttypecomment())
         if self.hastypecomment(typecomment, typecomments) != present:
             if present:
@@ -620,7 +679,7 @@ class pounit(pocommon.pounit):
     def markfuzzy(self, present=True):
         if present:
             self.set_state_n(self.STATE[self.S_FUZZY][0])
-        elif self.hasplural() and not self._msgstrlen() or is_null(self.msgstr):
+        elif (self.hasplural() and not self._msgstrlen()) or is_null(self.msgstr):
             self.set_state_n(self.STATE[self.S_UNTRANSLATED][0])
         else:
             self.set_state_n(self.STATE[self.S_TRANSLATED][0])
@@ -639,19 +698,19 @@ class pounit(pocommon.pounit):
         return self.obsolete
 
     def makeobsolete(self):
-        """Makes this unit obsolete"""
+        """Makes this unit obsolete."""
         super().makeobsolete()
         self.obsolete = True
         self.sourcecomments = []
         self.automaticcomments = []
 
     def resurrect(self):
-        """Makes an obsolete unit normal"""
+        """Makes an obsolete unit normal."""
         super().resurrect()
         self.obsolete = False
 
     def hasplural(self):
-        """returns whether this pounit contains plural strings..."""
+        """Returns whether this pounit contains plural strings..."""
         return len(self.msgid_plural) > 0
 
     def _getmsgpartstr(self, partname, partlines, partcomments=""):
@@ -659,7 +718,7 @@ class pounit(pocommon.pounit):
             partkeys = sorted(partlines.keys())
             return "".join(
                 self._getmsgpartstr(
-                    "%s[%d]" % (partname, partkey), partlines[partkey], partcomments
+                    f"{partname}[{partkey}]", partlines[partkey], partcomments
                 )
                 for partkey in partkeys
             )
@@ -683,13 +742,11 @@ class pounit(pocommon.pounit):
                 combinedcomment = []
                 for comment in partcomments:
                     comment = unquotefrompo([comment])
-                    if comment.startswith("_:"):
-                        comment = comment[len("_:") :]
-                    if comment.endswith("\\n"):
-                        comment = comment[: -len("\\n")]
+                    comment = comment.removeprefix("_:")
+                    comment = comment.removesuffix("\\n")
                     # Before we used to strip. Necessary in some cases?
                     combinedcomment.append(comment)
-                partcomments = self.quote("_:%s" % "".join(combinedcomment))
+                partcomments = self.quote("_:{}".format("".join(combinedcomment)))
                 # Strip heading empty line for multiline string, it was already added above
                 if partcomments[0] == '""':
                     partcomments = partcomments[1:]
@@ -713,7 +770,7 @@ class pounit(pocommon.pounit):
         return self._getoutput()
 
     def _getoutput(self):
-        """return this po element as a string"""
+        """Return this po element as a string."""
 
         def add_prev_msgid_lines(lines, prefix, header, var):
             if var:
@@ -751,9 +808,10 @@ class pounit(pocommon.pounit):
         # if there's no msgid don't do msgid and string, unless we're the
         # header this will also discard any comments other than plain
         # othercomments...
-        if is_null(self.msgid):
-            if not (self.isheader() or self.getcontext() or self.sourcecomments):
-                return "".join(lines)
+        if is_null(self.msgid) and not (
+            self.isheader() or self.getcontext() or self.sourcecomments
+        ):
+            return "".join(lines)
         lines.extend(self.automaticcomments)
         lines.extend(self.sourcecomments)
         lines.extend(self.typecomments)
@@ -771,7 +829,8 @@ class pounit(pocommon.pounit):
         return "".join(lines)
 
     def getlocations(self):
-        """Get a list of locations from sourcecomments in the PO unit
+        """
+        Get a list of locations from sourcecomments in the PO unit.
 
         rtype: List
         return: A list of the locations with '#: ' stripped
@@ -785,7 +844,8 @@ class pounit(pocommon.pounit):
         return locations
 
     def addlocation(self, location):
-        """Add a location to sourcecomments in the PO unit
+        """
+        Add a location to sourcecomments in the PO unit.
 
         :param location: Text location e.g. 'file.c:23' does not include #:
         :type location: String
@@ -795,20 +855,20 @@ class pounit(pocommon.pounit):
         self.sourcecomments.append(f"#: {location}{self.newline}")
 
     def _extract_msgidcomments(self, text=None):
-        """Extract KDE style msgid comments from the unit.
+        """
+        Extract KDE style msgid comments from the unit.
 
         :rtype: String
         :return: Returns the extracted msgidcomments found in this
                  unit's msgid.
         """
-
         if not text:
             text = unquotefrompo(self.msgidcomments)
         return text.split(self.newline)[0].replace("_: ", "", 1)
 
     def setmsgidcomment(self, msgidcomment):
         if msgidcomment:
-            self.msgidcomments = ['"_: %s\\n"' % msgidcomment]
+            self.msgidcomments = [f'"_: {msgidcomment}\\n"']
         else:
             self.msgidcomments = []
 
@@ -838,7 +898,7 @@ class pounit(pocommon.pounit):
 
 
 class pofile(pocommon.pofile):
-    """A .po file containing various units"""
+    """A .po file containing various units."""
 
     UnitClass = pounit
 
@@ -867,8 +927,9 @@ class pofile(pocommon.pofile):
         poparser.parse_units(poparser.ParseState(iter(lines), self.create_unit), self)
 
     def removeduplicates(self, duplicatestyle="merge"):
-        """Make sure each msgid is unique ; merge comments etc from
-        duplicates into original
+        """
+        Make sure each msgid is unique ; merge comments etc from
+        duplicates into original.
         """
         # TODO: can we handle consecutive calls to removeduplicates()? What
         # about files already containing msgctxt? - test
@@ -879,7 +940,9 @@ class pofile(pocommon.pofile):
         markedpos = []
 
         def addcomment(thepo):
-            thepo.msgidcomments.append('"_: %s\\n"' % " ".join(thepo.getlocations()))
+            thepo.msgidcomments.append(
+                '"_: {}\\n"'.format(" ".join(thepo.getlocations()))
+            )
             markedpos.append(thepo)
 
         for thepo in self.units:
@@ -896,20 +959,19 @@ class pofile(pocommon.pofile):
                         uniqueunits.append(thepo)
                 elif duplicatestyle == "msgctxt":
                     origpo = id_dict[id]
-                    if origpo not in markedpos and id:
-                        # if it doesn't have an id, we already added msgctxt
+                    if origpo not in markedpos and not origpo.msgctxt:
                         origpo.msgctxt.append(
-                            '"%s"' % escapeforpo(" ".join(origpo.getlocations()))
+                            '"{}"'.format(escapeforpo(" ".join(origpo.getlocations())))
                         )
                         markedpos.append(thepo)
                     thepo.msgctxt.append(
-                        '"%s"' % escapeforpo(" ".join(thepo.getlocations()))
+                        '"{}"'.format(escapeforpo(" ".join(thepo.getlocations())))
                     )
-                    if not thepo.msgctxt == id_dict[id].msgctxt:
+                    if thepo.msgctxt != id_dict[id].msgctxt:
                         uniqueunits.append(thepo)
                     else:
                         logger.warning(
-                            "Duplicate unit found with msgctx of '%s' and source '%s'",
+                            "Duplicate unit found with msgctxt of '%s' and source '%s'",
                             thepo.msgctxt,
                             thepo.source,
                         )
@@ -919,14 +981,14 @@ class pofile(pocommon.pofile):
                         addcomment(thepo)
                     else:
                         thepo.msgctxt.append(
-                            '"%s"' % escapeforpo(" ".join(thepo.getlocations()))
+                            '"{}"'.format(escapeforpo(" ".join(thepo.getlocations())))
                         )
                 id_dict[id] = thepo
                 uniqueunits.append(thepo)
         self.units = uniqueunits
 
     def serialize(self, out):
-        """Write to file"""
+        """Write to file."""
         at_start = True
         try:
             for unit in self.units:
@@ -949,5 +1011,8 @@ class pofile(pocommon.pofile):
                 yield unit
 
     def addunit(self, unit):
+        needs_update = unit.wrapper != self.wrapper
         unit.wrapper = self.wrapper
         super().addunit(unit)
+        if needs_update:
+            unit.target = unit.target
